#include "asterisk.h"
#include <math.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include <iconv.h>

#include <dahdi/user.h>
#include <dahdi/tonezone.h>

#include "asterisk/lock.h"
#include "asterisk/logger.h"
#include  "asterisk/channel.h"
#include "asterisk/app.h"
#include "asterisk/strings.h"
#include "asterisk/cli.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include "asterisk/manager.h"


#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xinclude.h>

#include "asterisk/manager.h"
#include "asterisk/sms.h"


const char *g_logpath="/var/log/asterisk/sms";
const char *g_sms_sendpath="/var/log/asterisk/sms/sendsms";
const char *g_sms_recvpath="/var/log/asterisk/sms/recvsms";

char oldsendlogfile[80];
char oldrecvlogfile[80];

static FILE *fp;


int ast_xml_init(void)
{
	LIBXML_TEST_VERSION

	return 0;
}

int ast_xml_finish(void)
{
	xmlCleanupParser();

	return 0;
}

struct ast_xml_doc *ast_xml_open(char *filename)
{
	xmlDoc *doc;

	if (!filename) {
		return NULL;
	}

	doc = xmlReadFile(filename, NULL, XML_PARSE_RECOVER);
	if (doc) {
		/* process xinclude elements. */
		if (xmlXIncludeProcess(doc) < 0) {
			xmlFreeDoc(doc);
			return NULL;
		}
	}

	return (struct ast_xml_doc *) doc;
}

void ast_xml_close(struct ast_xml_doc *doc)
{
	if (!doc) {
		return;
	}

	xmlFreeDoc((xmlDoc *) doc);
	doc = NULL;
}


struct ast_xml_node *ast_xml_get_root(struct ast_xml_doc *doc)
{
	xmlNode *root_node;

	if (!doc) {
		return NULL;
	}

	root_node = xmlDocGetRootElement((xmlDoc *) doc);

	return (struct ast_xml_node *) root_node;
}

void ast_xml_free_node(struct ast_xml_node *node)
{
	if (!node) {
		return;
	}

	xmlFreeNode((xmlNode *) node);
	node = NULL;
}

void ast_xml_free_attr(const char *attribute)
{
	if (attribute) {
		xmlFree((char *) attribute);
	}
}

void ast_xml_free_text(const char *text)
{
	if (text) {
		xmlFree((char *) text);
	}
}

const char *ast_xml_get_attribute(struct ast_xml_node *node, const char *attrname)
{
	xmlChar *attrvalue;

	if (!node) {
		return NULL;
	}

	if (!attrname) {
		return NULL;
	}

	attrvalue = xmlGetProp((xmlNode *) node, (xmlChar *) attrname);

	return (const char *) attrvalue;
}

struct ast_xml_node *ast_xml_find_element(struct ast_xml_node *root_node, const char *name, const char *attrname, const char *attrvalue)
{
	struct ast_xml_node *cur;
	const char *attr;

	if (!root_node) {
		return NULL;
	}

	for (cur = root_node; cur; cur = ast_xml_node_get_next(cur)) {
		/* Check if the name matchs */
		if (strcmp(ast_xml_node_get_name(cur), name)) {
			continue;
		}
		/* We need to check for a specific attribute name? */
		if (!attrname || !attrvalue) {
			return cur;
		}
		/* Get the attribute, we need to compare it. */
		if ((attr = ast_xml_get_attribute(cur, attrname))) {
			/* does attribute name/value matches? */
			if (!strcmp(attr, attrvalue)) {
				ast_xml_free_attr(attr);
				return cur;
			}
			ast_xml_free_attr(attr);
		}
	}

	return NULL;
}


const char *ast_xml_get_text(struct ast_xml_node *node)
{
	if (!node) {
		return NULL;
	}

	return (const char *) xmlNodeGetContent((xmlNode *) node);
}

const char *ast_xml_node_get_name(struct ast_xml_node *node)
{
	return (const char *) ((xmlNode *) node)->name;
}

struct ast_xml_node *ast_xml_node_get_children(struct ast_xml_node *node)
{
	return (struct ast_xml_node *) ((xmlNode *) node)->children;
}

struct ast_xml_node *ast_xml_node_get_next(struct ast_xml_node *node)
{
	return (struct ast_xml_node *) ((xmlNode *) node)->next;
}

struct ast_xml_node *ast_xml_node_get_prev(struct ast_xml_node *node)
{
	return (struct ast_xml_node *) ((xmlNode *) node)->prev;
}

struct ast_xml_node *ast_xml_node_get_parent(struct ast_xml_node *node)
{
	return (struct ast_xml_node *) ((xmlNode *) node)->parent;
}




struct ast_xml_doc *ast_xml_open_utf8(char *filename)
{
        xmlDoc *doc;
 
        if (!filename) {
                return NULL;
        }
 
        doc = xmlReadFile(filename, "UTF-8", XML_PARSE_RECOVER);
        if (doc) {
                /* process xinclude elements. */
                if (xmlXIncludeProcess(doc) < 0) {
                        xmlFreeDoc(doc);
                        return NULL;
                }
        }
 
        return (struct ast_xml_doc *) doc;
}



/*create sms path directory*/
int sms_create_log_path()
{
     DIR *pDir = NULL;
     struct dirent *pDirent = NULL;
     int res = -1;
     int my_umask;
     struct stat status;
    // if((pDir = opendir((char*)g_logpath)) == NULL) jwj change 1221
     if(0 != stat((char*)g_logpath,&status))
     {
         switch(errno)
         {
             case ENOENT: 
                my_umask=umask(0);
                umask(0);
                res = mkdir((char *)g_logpath,S_IRWXU|S_IRWXG|S_IRWXO);
                break;
             default:
                ast_log(LOG_WARNING,"crate sms log path find a unkown error\n");
                break;
         }
          
         return res;
     }
     else
     {
         //closedir(pDir);
         return 0;
     }
}

int sms_create_sendlog_path()
{
     DIR *pDir = NULL;
     struct dirent *pDirent = NULL;
     int res = -1;
     struct stat status;
     int my_umask;
     res = sms_create_log_path();
     if(res)
     {
         ast_log(LOG_WARNING,"sms create sendlog path fail\n");
         return -1;
     }
     
     res = -1 ;
     //if((pDir = opendir((char*)g_sms_sendpath)) == NULL)//jwj change 1221
     if( 0 != stat((char *)g_sms_sendpath, &status))
     {
         switch(errno)
         {
             case ENOENT:
                my_umask=umask(0);
                umask(0);
                res = mkdir((char *)g_sms_sendpath,S_IRWXU|S_IRWXG|S_IRWXO);
                break;
             default:
                ast_log(LOG_WARNING,"crate sendsms log path find a unkown error\n");
                break;
         }
         return res;
     }
     else
     {
        // closedir(pDir);
         return 0;
     }
}


int sms_create_recvlog_path()
{
     DIR *pDir = NULL;
     struct dirent *pDirent = NULL;
     int res = -1;
     char formattime[100]={0};
     time_t timer;
     char tmppath[100]={0};
     struct stat status;
     int my_umask;
     res = sms_create_log_path();
     if(res)
     {
         ast_log(LOG_WARNING,"sms create recvlog path fail\n");
         return -1;
     }
     
     res = -1 ;

     timer = time(NULL);
     strftime(formattime,sizeof(formattime) - 1,"%m-%d",localtime(&timer));//jwj change 1215
     snprintf(tmppath,sizeof(tmppath)-1,"%s_%s",g_sms_recvpath,formattime);
     

     //if((pDir = opendir((char*)g_sms_recvpath)) == NULL)
     //if((pDir = opendir((char*)tmppath)) == NULL)//jwj change 1221
     if( 0 != stat((char*)tmppath,&status))
     {
         switch(errno)
         {
             case ENOENT:
                my_umask=umask(0);
                umask(0);
                res = mkdir((char *)tmppath,S_IRWXU|S_IRWXG|S_IRWXO);
                break;
             default:
                ast_log(LOG_WARNING,"crate recvsms log path find a unkown error\n");
                break;
         }
         return res;
     }
     else
     {
         //closedir(pDir);
         return 0;
     }
}
/*
format:
time is localtime,send msg time
*/
int sms_write_sendlog(char *sessionid,int channum,char *recver,int res)
{ 
    time_t timer;
    char formattime[100]={0};
    //char logtime[100]={0};
    int result = 0;
 
    timer = time(NULL);
    strftime(formattime,sizeof(formattime) - 1,"%Y-%m-%d",localtime(&timer));

    ast_mutex_lock(&g_logfile_lock);
    result = sms_open_sendlog_file(formattime);
    if(result)
    {
       ast_log(LOG_WARNING,"sms write send log,but open file fail\n");
       ast_mutex_unlock(&g_logfile_lock);
       return -1;
    }
    
    strftime(formattime,sizeof(formattime) - 1,"%Y-%m-%d-%H-%M-%S",localtime(&timer));

    fprintf(fp, "channo:%d << %s|%s|%s|%s",channum, sessionid, formattime, recver,res>0?"success":"fail");
    fprintf(fp, "\n");
    fflush(fp);
    fclose(fp);
    ast_mutex_unlock(&g_logfile_lock);

    return 0;
}

FILE * sms_open_recvlog_file(int channo,  FILE *filep,char *filename)
{ 
     time_t timer;
     char formattime[100]={0};
     //char filename[80] = {0};
     int res = 0;     
     int my_umask;
     res = sms_create_recvlog_path();
     if(res)
     {
        ast_log(LOG_WARNING,"sms can not create log path\n");
        return NULL;
     }
     
     #if 0
     timer = time(NULL);
     strftime(formattime,sizeof(formattime) - 1,"%Y-%m-%d-%H-%M-%S",localtime(&timer));

     snprintf(filename,sizeof(filename),"%s/%s_%d_%s_%d%s",g_sms_recvpath,"recvsms",channo,formattime,flag,".log");
     #endif
 
     if( !filep )
     {
        
        my_umask=umask(0);
        umask(0);
        filep = fopen(filename, "a+");
        if(!filep)
        {
           ast_log(LOG_WARNING,"sms open recvlog file=%s fail recvpath=%s\n",filename,g_sms_recvpath);
           return NULL;
        }
        ast_log(LOG_WARNING,"sms open recvlog file=%s ok  recvpath=%s\n",filename,g_sms_recvpath);
     }
         
     return filep;
}


/*
format
time is pdu msg time,not localtime

*/
int sms_write_recvlog(int channum, char *sender, char *timer, char *content,char* longsms_flag)
{
    FILE *filep = NULL;
    char filename[80] = {0};
    time_t tm;
    char formattime[100]={0};

    char tmppath[100]={0};
    char tmptime[20]={0};    
  
     #if 1
     tm = time(NULL);
     //strftime(formattime,sizeof(formattime) - 1,"%Y-%m-%d-%H-%M-%S",localtime(&tm));
     strftime(formattime,sizeof(formattime) - 1,"%H%M%S",localtime(&tm));
     strftime(tmptime,sizeof(tmptime) - 1,"%m-%d",localtime(&tm));//jwj add 1215
     snprintf(tmppath,sizeof(tmppath)-1,"%s_%s",g_sms_recvpath,tmptime);
     #endif

    if(longsms_flag[0]=='\0')
    snprintf(filename,sizeof(filename),"%s/%s_%d_%s_%s%s",tmppath,"recvsms",channum,formattime,sender,".log");
    else
    snprintf(filename,sizeof(filename),"%s/%s_%d_%s_%s_%s%s",tmppath,"recvsms",channum,formattime,sender,longsms_flag,".log");
    
    filep = sms_open_recvlog_file(channum, filep,filename);

    if(!filep)
    {
       ast_log(LOG_WARNING,"sms write recv log,but open logfile fail\n");
       return -1;
    }
    
    fprintf(filep, "channo:%d>>%s|%s|%s",channum, timer, sender, content);
    fprintf(filep, "\n");
    fflush(filep);
    fclose(filep);

    /*add manager event*/
    manager_event(EVENT_FLAG_CALL, "newsms",
                              "filename: %s\r\n",
                              "channum: %d\r\n",
                              "time: %s\r\n",
                              "sender: %s\r\n",filename,channum,timer,sender);
  
    return 0;
}
int sms_open_sendlog_file(char *formatime)
{
     char filename[80] = {0}; 
     int res = 0;
     int my_umask;   

     res = sms_create_sendlog_path(); 
     if(res)
     {
        ast_log(LOG_WARNING,"sms create sendpath fail\n");
        return -1;
     }
     snprintf(filename,sizeof(filename),"%s/%s_%s%s",g_sms_sendpath,"sendsms",formatime,".log");
      
     //if(strcasecmp(filename,oldlogsendfile)|| !fp )
     {
        if(fp)
           close(fp);
        
        my_umask=umask(0);
        umask(0);
        fp = fopen(filename, "a+");
        if(!fp)
        {
            fp = fopen(filename, "a+");
            if(!fp)
            {
               ast_log(LOG_WARNING,"sms open sendlog file=%s fail\n",filename);
               return -1;
            }
        }
        
        ast_log(LOG_WARNING,"sms open sendlog file ok and copyto oldsendlogfile=%s newfile=%s\n",oldsendlogfile,filename);
        memcpy(oldsendlogfile,filename,sizeof(oldsendlogfile));
     }
    return 0;
}

int map_language_to_id(char *lang,int *id)
{
   int res;
   if(!strcasecmp(lang,"cn"))
      *id = 0;
   else if(!strcasecmp(lang,"weur")) 
      *id = 1;
   else if(!strcasecmp(lang,"eeur"))
      *id = 2;
   else
      *id = 1;
   return res;
}


int sms_decode_language(char *in, size_t inlen, char *out, size_t outlen, int id)
{
	unsigned char tmp[512] = {0};
	int outleft;
	int i;
	int j;
        int languae_id;
 
	tmp[0] = 0xFF;
	tmp[1] = 0xFE;//file format utf-16
	j = 2 + inlen;

	for (i = 0; ((i < inlen)); i += 2) {
		tmp[i + 2] = in[i + 1];
		tmp[i + 3] = in[i];
	}

       
        switch(id)
        {
           case 0://cn
              ast_log(LOG_WARNING,"encode language id=%d is gb18030\n",id);
	      outleft = unicode_2_gb18030(tmp, j, out, outlen);
              break;
           case 1: //western eur
              ast_log(LOG_WARNING,"encode language id=%d is iso_8859-1\n",id);
	      outleft = unicode_2_weur(tmp, j, out, outlen);
              break;
           case 2://eastern eur
              ast_log(LOG_WARNING,"encode language id=%d is iso_8859-2\n",id);
	      outleft = unicode_2_eeur(tmp, j, out, outlen);
              break;
           default://en ascii
              ast_log(LOG_WARNING,"encode language id=%d is iso_8859-1\n",id);
	      outleft = unicode_2_weur(tmp, j, out, outlen);
              break;
        }
        
	if (outleft <= 0) {
		return -1;
	}

	out[outlen-outleft] = '\0';

	return outlen - outleft;
}

int unicode_2_eeur(unsigned char *inbuf,size_t inlen,unsigned char *outbuf,size_t outlen)
{
  return code_convert("unicode","ISO_8859-2",inbuf,inlen,outbuf,outlen);
}

int unicode_2_gb18030(char *inbuf,size_t inlen,char *outbuf,size_t outlen)
{
  return code_convert("unicode","gb18030",inbuf,inlen,outbuf,outlen);
}


int unicode_2_weur(char *inbuf,size_t inlen,char *outbuf,size_t outlen)
{
  return code_convert("unicode","ISO_8859-1",inbuf,inlen,outbuf,outlen);
}


int unicode_2_en(char *inbuf,size_t inlen,char *outbuf,size_t outlen)
{
  return code_convert("unicode","ISO_8859-1",inbuf,inlen,outbuf,outlen);
}

int code_convert(char *from_charset,char *to_charset,unsigned char *inbuf,size_t inlen,unsigned char *outbuf,size_t outlen)
{
	iconv_t cd;
	unsigned char **pin = &inbuf;
	unsigned char **pout = &outbuf;

	cd = iconv_open(to_charset,from_charset);
	if (cd==0) {
                ast_log(LOG_WARNING,"iconv_open error return\n");
		return -1;
	}

	memset(outbuf,0,outlen);

	if (iconv(cd,pin,&inlen,pout,&outlen)==-1) {
                ast_log(LOG_WARNING,"iconv error and return,inlen=%d outlen=%d from=%s to=%s errno=%d in[0]=0x%x in[1]=0x%x\n",inlen,outlen,from_charset,to_charset,errno,inbuf[0],inbuf[1]);
	        iconv_close(cd);
		return -1;
	}

	iconv_close(cd);

	return outlen;
}


unsigned long gsm_hex_to_int(char *src_hex, int len)
{
      int i,j;
      unsigned long res = 0;

      for (i=0; i<len; i++) {
	j = 0;
	if ((src_hex[i] >= 48) && (src_hex[i] <=57)) { /* '0' - '9' */
		j = src_hex[i] - 48;
	} else if ((src_hex[i] >= 65) && (src_hex[i] <=70)) { /* 'A' - 'F' */
		j = src_hex[i] - 55;
	} else if ((src_hex[i] >= 97) && (src_hex[i] <=102)) { /* 'a' - 'f' */
		j = src_hex[i] - 87;
	}

	res += j * (unsigned long)pow(16, len - i - 1);
      }
      return res;
}

void gsm_ucs2_to_byte(unsigned char *dest, char *src, int len)
{
	int i;

	for (i = 0; ( (i < len) && (src[i] != '\0') ); i++) {
		dest[i] = (unsigned char)gsm_hex_to_int(&src[i*2], 2);
	}
	dest[len * 2] = '\0';
}

char *pdu_get_smsc_len(char *src, char *len)
{
	char *p_start, *p_end;

	p_start = src;
	p_end = p_start + 2;

	*len = (char)gsm_hex_to_int(p_start, 2);
       return p_end;
}

void gsm_reverse_num(char *src, char *dest, int type)
{
	if (type == 0x91) {
		src[0] = '+';
		gsm_reverse_string(src+1, dest);
	} else {
		gsm_reverse_string(src, dest);
	}
}

char *pdu_get_first_byte(char *src, char *first_byte)
{
	char *p_start, *p_end;
	
	p_start = src;
	p_end = p_start + 2;
		
	*first_byte = (char)gsm_hex_to_int(p_start, 2);

	return p_end;	
}

char *pdu_get_oa_len(char *src, char *len)
{
	char *p_start, *p_end;
	
	p_start = src;
	p_end = p_start + 2;
		
	*len = (char)gsm_hex_to_int(p_start, 2);

	return p_end;	
}

char *pdu_get_oa_type(char *src, char *type)
{
	char *p_start, *p_end;
	
	p_start = src;
	p_end = p_start + 2;
		
	*type = (char)gsm_hex_to_int(p_start, 2);

	return p_end;	
}

char *pdu_get_oa_num(char *src, int len, char *num)
{
	char *p_start, *p_end;

	p_start = src;

	if (len % 2) {
		len++;
	}
	p_end = p_start + len;

	strncpy(num, p_start,len);

	num[len] = '\0';

	return p_end;
}

char *pdu_get_pro_flag(char *src, char *pro_flag)
{
	char *p_start, *p_end;
	
	p_start = src;
	p_end = p_start + 2;
		
	*pro_flag = (char)gsm_hex_to_int(p_start, 2);

	return p_end;	
}

char *pdu_get_ud_coding(char *src, char *code)
{
	char *p_start, *p_end;
	
	p_start = src;
	p_end = p_start + 2;
		
	*code = (char)gsm_hex_to_int(p_start, 2);

	return p_end;	
}

char *pdu_get_timestamp(char *src, char *tstamp)
{
	char *p_start, *p_end;
	int len = 14;
	
	p_start = src;
	p_end = p_start + len;
	
	strncpy(tstamp, p_start, len);
	tstamp[len] = '\0';

	return p_end;
}

char *pdu_get_ud_len(char *src, unsigned char *len)
{
	char *p_start, *p_end;
	
	p_start = src;
	p_end = p_start + 2;
		
	*len = (unsigned char)gsm_hex_to_int(p_start, 2);

	return p_end;	
}

void gsm_reverse_string(char *conv_dest, char *src)
{
	int i;
	int len = strlen(src);//
        for(i=0; i < len; i+=2) {
		conv_dest[i] = src[i+1];
		if ((src[i] != 'F') && (src[i] != 'f')) {
				conv_dest[i+1] = src[i];
		} else {
				conv_dest[i+1] = '\0';
				return;
		} 
	}

	if (i >= len) {
		conv_dest[i] = '\0';
	}
}

char *pdu_get_smsc_num(char *src, int len, char *dest)
{
	char *p_start, *p_end;
	
	int num_len = (len - 1) * 2;		
	p_start = src;
	p_end = p_start + num_len;
	
	strncpy(dest, p_start, num_len);
	dest[num_len] = '\0';

	return p_end;
}

char *pdu_get_smsc_type(char *src, char *type)
{
	char *p_start, *p_end;

	p_start = src;
	p_end = p_start + 2;
		
	*type = (char)gsm_hex_to_int(p_start, 2);

	return p_end;
}

int  sms_pdu_decode(const unsigned char* pSrc,struct gsm_pdu_msg *pdu_msg)
{
   int len;

   char smsc_num[32];
   char oa_num[32];
   char timestamp[16];
   char *tmp = NULL;
   
   len = strlen(pSrc);

   if (len < 14){
	return -1;
   }
   
   tmp = pSrc;
   memset(pdu_msg, 0, sizeof(struct gsm_pdu_msg));
   
   tmp = pdu_get_smsc_len(tmp, (char*)&pdu_msg->smsc_num_len);
   
   if(pdu_msg->smsc_num_len > 0)
   {
      tmp  = pdu_get_smsc_type(tmp, (char*)&pdu_msg->smsc_num_type);
      if ((pdu_msg->smsc_num_len - 1) * 2 >= sizeof(smsc_num)) {
          return -1;
      }
      tmp = pdu_get_smsc_num(tmp,pdu_msg->smsc_num_len,smsc_num);
      gsm_reverse_num((char *)pdu_msg->smsc_num,smsc_num,pdu_msg->smsc_num_type);
     
   }
 
   tmp = pdu_get_first_byte(tmp,(char *)&pdu_msg->first_byte);
   tmp = pdu_get_oa_len(tmp,(char *)&pdu_msg->oa.len);
   if (pdu_msg->oa.len > 0) {
     tmp = pdu_get_oa_type(tmp, (char *)&pdu_msg->oa.type);
     if (pdu_msg->oa.len + 1 >= sizeof(oa_num)) {
        return -1;
     }
     tmp = pdu_get_oa_num(tmp,pdu_msg->oa.len,oa_num);
     gsm_reverse_num(pdu_msg->oa.num,oa_num,pdu_msg->oa.type);
   }

   tmp = pdu_get_pro_flag(tmp,(char *)&pdu_msg->pro_flag);
   tmp = pdu_get_ud_coding(tmp,(char *)&pdu_msg->ud_coding);
   tmp = pdu_get_timestamp(tmp, timestamp);
   gsm_reverse_string(pdu_msg->timestamp,timestamp);
   
   tmp = pdu_get_ud_len(tmp,&pdu_msg->ud_len);
   
   ast_copy_string(pdu_msg->ud,tmp,sizeof(pdu_msg->ud)); 

   return 0;
}

int sms_pud_ucs2_to_utf8(const char *pSrc, unsigned char *pDst)
{
   return 0;
}

int sms_save_recv_msg(const char *pSrc)
{
    return 0;
}

int  gsmEncode7bit(const  char* pSrc, unsigned char* pDst, int nSrcLength)
{   
  int   nSrc;
  int   nDst;
  int   nChar;
  unsigned char nLeft;
  
  nSrc = 0;
  nDst = 0;
  
  if(nSrcLength == 1)
  {
    *pDst = *pSrc;
    return 1;
  }

  while(nSrc < nSrcLength)
  {
     nChar = nSrc & 7;//0-7
     if(nChar == 0)
     {
       nLeft   =   *pSrc;//
     }
     else
     {
       *pDst = (*pSrc << (8-nChar))|nLeft;
       nLeft = *pSrc >> nChar;

       pDst++;
       nDst++;
     }
     pSrc++;
     nSrc++;
  }

  if(nChar != 7)
   *pDst = nLeft;

  return nDst;
}

int  gsmDecode7bit(const unsigned char* pSrc, char* pDst, int nSrcLength)
{
  int   nSrc;
  int   nDst;
  int   nByte;
  unsigned char nLeft;

  nSrc = 0;
  nDst = 0;
  
  nByte = 0;
  nLeft = 0;

  while(nSrc < nSrcLength)
  {
    *pDst = ((*pSrc << nByte)| nLeft) & 0x7f;

    nLeft = *pSrc >> (7-nByte);
    pDst++;
    nDst++;
    nByte++;

    if(nByte == 7)
    {
       *pDst = nLeft;
       pDst++;
       nDst++;
       nByte = 0;
       nLeft = 0;
    }
    pSrc++;
    nSrc++;
  }
  *pDst   =   '/0';   
  return   nDst;
}

void gsm_to8Bit(unsigned char *in, unsigned char *out, int len)
{
	int i = 0;
	int inputOffset = 0;
	int outputOffset = 0;

	while (inputOffset < len ) {
	out[outputOffset] = (in[inputOffset] & (unsigned char)(pow(2, 7-i)-1));
	if (i == 8) {
		out[outputOffset] = in[inputOffset-1] & 127;
		i = 1;
	} else {
		out[outputOffset] = out[outputOffset] << i;
		out[outputOffset] |= in[inputOffset-1] >> (8-i);
		inputOffset++;
		i++;
	}
		outputOffset++;
	}

	out[len] = '\0';
}

/* \brief callback function sendtext for test will be remove */
 long utf8_to_ucs2(unsigned char **pp)
{
	unsigned char *p = *pp;
	if (!*p) {
		return 0;                           /* null termination of string */
	}
	(*pp)++;
	if (*p < 0xC0) {
		return *p;                          /* ascii or continuation character */
	}
    
	if (*p < 0xE0) {
		if (*p < 0xC2 || (p[1] & 0xC0) != 0x80) {
			return *p;                      /* not valid UTF-8 */
		}
		(*pp)++;
		return ((*p & 0x1F) << 6) + (p[1] & 0x3F);//
   	}
	if (*p < 0xF0) {
		if ((*p == 0xE0 && p[1] < 0xA0) || (p[1] & 0xC0) != 0x80 || (p[2] & 0xC0) != 0x80) {
			return *p;                      /* not valid UTF-8 */
		}
		(*pp) += 2;
		return ((*p & 0x0F) << 12) + ((p[1] & 0x3F) << 6) + (p[2] & 0x3F);
	}
	if (*p < 0xF8) {
		if ((*p == 0xF0 && p[1] < 0x90) || (p[1] & 0xC0) != 0x80 || (p[2] & 0xC0) != 0x80 || (p[3] & 0xC0) != 0x80) {
			return *p;                      /* not valid UTF-8 */
		}
		(*pp) += 3;
		return ((*p & 0x07) << 18) + ((p[1] & 0x3F) << 12) + ((p[2] & 0x3F) << 6) + (p[3] & 0x3F);
	}
	if (*p < 0xFC) {
		if ((*p == 0xF8 && p[1] < 0x88) || (p[1] & 0xC0) != 0x80 || (p[2] & 0xC0) != 0x80 || (p[3] & 0xC0) != 0x80
			|| (p[4] & 0xC0) != 0x80) {
			return *p;                      /* not valid UTF-8 */
		}
		(*pp) += 4;
		return ((*p & 0x03) << 24) + ((p[1] & 0x3F) << 18) + ((p[2] & 0x3F) << 12) + ((p[3] & 0x3F) << 6) + (p[4] & 0x3F);
	}
	if (*p < 0xFE) {
		if ((*p == 0xFC && p[1] < 0x84) || (p[1] & 0xC0) != 0x80 || (p[2] & 0xC0) != 0x80 || (p[3] & 0xC0) != 0x80
			|| (p[4] & 0xC0) != 0x80 || (p[5] & 0xC0) != 0x80) {
			return *p;                      /* not valid UTF-8 */
		}
		(*pp) += 5;
		return ((*p & 0x01) << 30) + ((p[1] & 0x3F) << 24) + ((p[2] & 0x3F) << 18) + ((p[3] & 0x3F) << 12) + ((p[4] & 0x3F) << 6) + (p[5] & 0x3F);
	}
	return *p;                              /* not sensible */
}


 int is_packsms8(unsigned char *o,int udl, unsigned short *ud)
{
    unsigned char p = 0;
    unsigned char dummy[1200] = {0};
    int flag = 0;
    
    if (o == NULL)
      o = dummy; 

    while (udl--)
    {
	long u;
	u = *ud++;
	if (u < 0 || u > 0x7F)
        {
            flag = 1;
	    //return -1;                      /* not valid */
	}
	o[p++] = u;
   }
   return flag;        
}


 int sms_check_senddata(unsigned char *data,unsigned short *ud)
{
   int ud_len = 0;
   int i= 0;
  
   while(*data != '\0')
   {
     ud[ud_len] = utf8_to_ucs2(&data);//utf-8 to ucs-2 
    // ast_log(LOG_WARNING,"utf8decode tmp=%x,len=%d\n",ud[ud_len],ud_len);
     ud_len++;
   }
   ud[ud_len]='\0';
   ast_log(LOG_WARNING,"sms_check_senddata is ucs2 code find ud_len=%d\n",ud_len);
   if(is_packsms8(0,ud_len,ud))
   {
      return ud_len;
   }
   
   return 0;
}

 int sms_pdu_convert_callednum(char *src_called, char *dest_called, int *called_len)
{
    int len = strlen(src_called);
    int i = 0;
    char ch;

    if(len > 16)
    return -1;

    ast_log(LOG_WARNING,"chan dahdi sms convert callednum=%s len=%d\n",src_called,len);
    for(i = 0; i < len; i+=2)
    {
        ch = *src_called++;//7
        *dest_called++ = *src_called++;//
        *dest_called++ = ch;//7 already move dest_called[len] not the first value
    }
    
    if(len&1)
    {
      *(dest_called-2) = 'F';
      *called_len =  len+2 ;//86
    }
    else 
     *called_len = len+2;

    // ={ and F
    *dest_called = '\0' ;
    
    return 0;
}

/*short to byte*/
 int pdu_ucs2_to_char(unsigned char *out,unsigned short *ud,int len)
{
    int p = 0;
    int i = 0;
   
    while(len--)
    {
         long u;
	 u = *ud++; 
        
	 out[p++] = (u >> 8);
  
         //ast_log(LOG_WARNING,"package ucs2 len=%d p=%d out=%c out=%x\n",len,p,out[p-1],out[p-1]);

         out[p++] = u;
         //ast_log(LOG_WARNING,"package ucs2 len=%d p=%d out=%c out=%x\n",len,p,out[p-1],out[p-1]);
    }
    
    out[p] = '\0';

    return p;
}


//called_len,conv_called,content
int sms_pdu_package(char *data,unsigned char *udata,struct pdu_package *pk,int pdu_mode)
{
    int total_len = 0;
    int content_len = 0;
    unsigned char save_data[50]={0};
    unsigned char tmp_data[50]={0};
    int i= 0;
    if(pdu_mode !=2 )
    {
          //if(pk->longflag)
           //  content_len = ((pk->send_len+12)*4)/7;
          //else
             content_len = (pk->send_len*4)/7;
    }

    total_len = 2 + pk->calledlen + 2 + pk->send_len/2 +  3 + 1; //1100+2len+96+008000
    
   // gsmEncode7bit("050003",save_data,strlen("050003"));
    // for(i = 0;i<strlen(save_data);i++)

    if(pk->longflag == 1)
    { 
        if(pdu_mode == 2)      
           snprintf(data,400,"%s%02X%s%s%s%02X%s%02X%02X%02X%s","005100",pk->calledlen,"9168",pk->conv_called,"000800",pk->send_len/2+6,"050003",pk->longsms_id,pk->totalnum,pk->smsnum,udata);//usc2 coding
        else
        //   snprintf(data,400,"%s%02X%s%s%s%02X%s%02X%02X%02X%s","005100",pk->calledlen,"9168",pk->conv_called,"000000",content_len,"050003",pk->longsms_id,pk->totalnum,pk->smsnum,udata);//usc2 coding
    
        snprintf(data,400,"%s%02X%s%s%s%02X%s","001100",pk->calledlen,"9168",pk->conv_called,"000000",content_len,udata);
    }
    else
    {
        if(pdu_mode == 2)
            snprintf(data,400,"%s%02X%s%s%s%02X%s","001100",pk->calledlen,"9168",pk->conv_called,"000800",pk->send_len/2,udata);//usc2 coding
        else
            snprintf(data,400,"%s%02X%s%s%s%02X%s","001100",pk->calledlen,"9168",pk->conv_called,"000000",content_len,udata);
    }
  ast_log(LOG_WARNING,"sms pdu package send content_len=0x%x,send_len=0x%x smsnum=%d mode=%d\n",content_len,pk->send_len,pk->smsnum,pdu_mode);
   return 0;
}

/*must set long sms flag send 51 recv 44 050003220202*/
int sms_pdu_7bit_package(char *data,unsigned char *udata,int send_len, int  calledlen, char *conv_called)
{
    int content_len = 0;
    content_len = (send_len*4)/7;
     
    snprintf(data,400,"%s%02X%s%s%s%02X%s","001100",calledlen,"9168",conv_called,"000000",content_len,udata);//7bit coding
    
   
   return 0;
}

/*maybe change*/
 int sms_convert_content_to_2hex(unsigned char *msg,int msg_len)
{
    char utmp[MAX_SMS_PDU_HEX_LEN] = {0};
    char content[MAX_SMS_PDU_HEX_LEN] = {0};
    int len = 0;
    int i = 0;

    #if 1
    for(i = 0;i<msg_len;i++)
    {
       
      #if 1
      if(i==0)
      {
         snprintf(content,sizeof(content),"%02X",msg[i]);
         ast_copy_string(utmp,content,sizeof(utmp));
      }
      else
      ast_copy_string(content,utmp,sizeof(content));

      if(i>0)
      {
         snprintf(utmp,sizeof(utmp),"%s%02X",content,msg[i]);
      }
      #endif
    }
    #endif

    len = strlen(utmp);
    
    memset(msg,0,MAX_SMS_PDU_HEX_LEN);
     
    if(len < MAX_SMS_PDU_HEX_LEN-1)
    ast_copy_string(msg,utmp,len+1);
     
    msg[len] = '\0' ;
    
    //ast_log(LOG_WARNING,"content_to hex msg=%s tmplen=%d msg_len=%d tmp=%s msglen=%d\n",msg,len,msg_len,utmp,strlen(msg));
    return 0;
}

 int dahdi_decode_pdu(void)
{
   return 0;
}

